import streamlit as st
from dash import MultiApp
from apps import summary, orders, customers,traffic,discounts,churn,product_rec,products # import your app modules here

app = MultiApp()

# Add all your application here
app.add_app("Summary stats", summary.app)
app.add_app("Orders data analysis", orders.app)
app.add_app("Customers data analysis", customers.app)
app.add_app('Paid traffic analysis', traffic.app)
app.add_app('Discount Analysis', discounts.app)
app.add_app('Customer churn', churn.app)
app.add_app('Product Recommendation', product_rec.app)
app.add_app('Products Analysis', products.app)
# The main app
app.run()


import pandas as pd
import numpy as np
import streamlit as st
import time
import plotly.express as px
import matplotlib.pyplot as plt
import json
import altair as alt
import getpass
# pwd = getpass.getpass("passsword:")
from toolkit_w.snowflake.snowflakeq import Snowflakeq
SQ = Snowflakeq()
st.title('Analytics Dashboard')

# Draw a title and some text to the app:
'''
Analytics Dashboard
'''
import streamlit as st

class MultiApp:
    """
    Framework for combining multiple streamlit applications.
    """
    def __init__(self):
        self.apps = []

    def add_app(self, title, func):
        """Adds a new application.
        Parameters
        ----------
        func:
            the python function to render this app.
        title:
            title of the app. Appears in the dropdown in the sidebar.
        """
        self.apps.append({
            "title": title,
            "function": func
        })

    def run(self):

        app = st.sidebar.selectbox(
            'Navigation',
            self.apps,
            format_func=lambda app: app['title'])


        # customer = st.sidebar.selectbox(
            # 'Design Partner',
            # ['SAFARI_LTD','DAILY_PAPER'])



        # app = st.sidebar.radio(
        #     'Go To',
        #     self.apps,
        #     format_func=lambda app: app['title'])

        app['function']()








import pandas as pd
import numpy as np
import json


class Cleaner:
    """ Module that helps to clean and arrange raw data"""

    def getCleanData(self, df: pd.DataFrame):
        """
        Params:
            df-> Dataframe
        -----------------------
        Output:
            Clean dataframe after filtering
            log_file - Description of actions taken
        """
        log_file = []
        # Clean cancelled orders
        df['cancelledat'] = df['cancelledat'].fillna('Valid')
        df_1 = df[(df.cancelledat == 'Valid')]
        # Record canceled orders
        canceled_orders_num = (len(df) - len(df_1))

        # Clean free orders
        free = df[(df.totaldiscounts == df.totalorderitemsprice) | (df.totalprice == 0)]
        free_orders_num = len(free)

        df_2 = df_1[~(df_1.id.isin(free['id'].values))]
        print(len(df_1), len(df_2))
        # Missing customer data

        missing_customers = len(df_1[(df_1['customerid'].isnull())])

        df_3 = df_2[~(df_2['customerid'].isnull())]
        print(len(df_2), len(df_3))
        log_file.append([canceled_orders_num, free_orders_num, missing_customers])

        # log file for documentation
        log_file = pd.DataFrame(log_file, columns=['Number_of_canceled_Orders', 'Number_of_free_orders',
                                                   'Number_of_missing_customers_info'])
        return df_3, log_file

    def getshippingprice(self, df):
        """

        """
        df['shipping_price'] = df['shippingaggregate'].apply(lambda x: Cleaner().getshippingpricevalue(x))

        return df

    def getshippingpricevalue(self, x):
        if len(x) > 1:
            try:
                value = json.loads(x)
            except:
                shipping_price = 'error'
            try:
                shipping_price = value[0]['discounted_price']
                shipping_price = pd.to_numeric(shipping_price)
            except:
                shipping_price = 'error'
        else:
            shipping_price = 0

        return shipping_price



import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
# set the sf account name
sf_account = 'ra45066.eu-west-1'
from toolkit_w.snowflake.snowflakeq import Snowflakeq
SQ = Snowflakeq()

@staticmethod
def make_confusion_matrix(cf,
                          group_names=None,
                          categories='auto',
                          count=True,
                          percent=True,
                          cbar=True,
                          xyticks=True,
                          xyplotlabels=True,
                          sum_stats=True,
                          figsize=None,
                          cmap='Blues',
                          title=None):
    '''
    This function will make a pretty plot of an sklearn Confusion Matrix cm using a Seaborn heatmap visualization.
    Arguments
    ---------
    cf:            confusion matrix to be passed in
    group_names:   List of strings that represent the labels row by row to be shown in each square.
    categories:    List of strings containing the categories to be displayed on the x,y axis. Default is 'auto'
    count:         If True, show the raw number in the confusion matrix. Default is True.
    normalize:     If True, show the proportions for each category. Default is True.
    cbar:          If True, show the color bar. The cbar values are based off the values in the confusion matrix.
                   Default is True.
    xyticks:       If True, show x and y ticks. Default is True.
    xyplotlabels:  If True, show 'True Label' and 'Predicted Label' on the figure. Default is True.
    sum_stats:     If True, display summary statistics below the figure. Default is True.
    figsize:       Tuple representing the figure size. Default will be the matplotlib rcParams value.
    cmap:          Colormap of the values displayed from matplotlib.pyplot.cm. Default is 'Blues'
                   See http://matplotlib.org/examples/color/colormaps_reference.html

    title:         Title for the heatmap. Default is None.
    '''

    # CODE TO GENERATE TEXT INSIDE EACH SQUARE
    blanks = ['' for i in range(cf.size)]

    if group_names and len(group_names) == cf.size:
        group_labels = ["{}\n".format(value) for value in group_names]
    else:
        group_labels = blanks

    if count:
        group_counts = ["{0:0.0f}\n".format(value) for value in cf.flatten()]
    else:
        group_counts = blanks

    if percent:
        group_percentages = ["{0:.2%}".format(value) for value in cf.flatten() / np.sum(cf)]
    else:
        group_percentages = blanks

    box_labels = [f"{v1}{v2}{v3}".strip() for v1, v2, v3 in zip(group_labels, group_counts, group_percentages)]
    box_labels = np.asarray(box_labels).reshape(cf.shape[0], cf.shape[1])

    # CODE TO GENERATE SUMMARY STATISTICS & TEXT FOR SUMMARY STATS
    if sum_stats:
        # Accuracy is sum of diagonal divided by total observations
        accuracy = np.trace(cf) / float(np.sum(cf))

        # if it is a binary confusion matrix, show some more stats
        if len(cf) == 2:
            # Metrics for Binary Confusion Matrices
            precision = cf[1, 1] / sum(cf[:, 1])
            recall = cf[1, 1] / sum(cf[1, :])
            f1_score = 2 * precision * recall / (precision + recall)
            stats_text = "\n\nAccuracy={:0.3f}\nPrecision={:0.3f}\nRecall={:0.3f}\nF1 Score={:0.3f}".format(
                accuracy, precision, recall, f1_score)
        else:
            stats_text = "\n\nAccuracy={:0.3f}".format(accuracy)
    else:
        stats_text = ""

    # SET FIGURE PARAMETERS ACCORDING TO OTHER ARGUMENTS
    if figsize == None:
        # Get default figure size if not set
        figsize = plt.rcParams.get('figure.figsize')

    if xyticks == False:
        # Do not show categories if xyticks is False
        categories = False

    # MAKE THE HEATMAP VISUALIZATION
    plt.figure(figsize=figsize)
    sns.heatmap(cf, annot=box_labels, fmt="", cmap=cmap, cbar=cbar, xticklabels=categories, yticklabels=categories)

    if xyplotlabels:
        plt.ylabel('True label')
        plt.xlabel('Predicted label' + stats_text)
    else:
        plt.xlabel(stats_text)

    if title:
        plt.title(title)


class Tellme:
    """ Module for internal analytics - different plots and exploratory data analysis"""

    def getAvgorders(self, data, max_date, min_date):
        """    """

        data['Avg_Days_section'] = pd.cut(x=data['avg_days_between_purchases'],
                                        bins=[-1, 10, 20, 30, 40, 50, 60, 70, 80, 90, 1000],
                                        labels=['0-10', '10-20', '20-30', '30-40', '40-50', '50-60', '60-70', '70-80',
                                                '80-90', '90+'])

        dat = data.groupby('Avg_Days_section').agg({'customerid':'count'})


        return dat








